utils

package
v0.0.0-...-e721fde Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 2, 2026 License: Apache-2.0 Imports: 2 Imported by: 0

Documentation

Overview

Package utils provides generic helpers that are regularly useful for a range of test authors. TODO: question the usefulness of this and whether this should even be part of tigron.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func RandomStringBase64

func RandomStringBase64(desiredLength int) string

RandomStringBase64 generates a base64 encoded random string.

Types

This section is empty.

Directories

Path Synopsis
Package testca provides helpers to create a self-signed CA certificate, and the ability to generate signed certificates from it.
Package testca provides helpers to create a self-signed CA certificate, and the ability to generate signed certificates from it.

Jump to